@@ -39,7 +39,10 @@ class SnippetData:
 			'accelerator': ''}
 
 	def __init__(self, node, library):
+		self.priv_id = node.attrib.get('id')
+
 		self.set_library(library)
+		self.valid = False
 		self.set_node(node)
 
 	def can_modify(self):
@@ -51,6 +54,8 @@ class SnippetData:
 		else:
 			self.library = None
 
+		self.id = NamespacedId(self.language(), self.priv_id).id
+
 	def set_node(self, node):
 		if self.can_modify():
 			self.node = node
@@ -64,7 +69,6 @@ class SnippetData:
 			return
 
 		self.override = node.attrib.get('override')
-		self.id = NamespacedId(self.language(), node.attrib.get('id')).id
 
 		self.properties = {}
 		props = SnippetData.PROPS.copy()
@@ -97,6 +101,18 @@ class SnippetData:
 				self.properties[prop] = child
 			else:
 				self.properties[prop] = props[prop]
+		
+		self.check_validation()
+	
+	def check_validation(self):
+		if not self['tag'] and not self['accelerator']:
+			return False
+
+		library = SnippetsLibrary()
+		keyval, mod = gtk.accelerator_parse(self['accelerator'])
+		
+		self.valid = library.valid_tab_trigger(self['tag']) and \
+				(not self['accelerator'] or library.valid_accelerator(keyval, mod))
 	
 	def __getitem__(self, prop):
 		if prop in self.properties:
@@ -131,6 +147,8 @@ class SnippetData:
 			if prop == 'tag' or prop == 'accelerator':
 				container = SnippetsLibrary().container(self.language())
 				container.prop_changed(self, prop, oldvalue)
+		
+		self.check_validation()
 
 	def language(self):
 		if self.library and self.library():
@@ -138,6 +156,9 @@ class SnippetData:
 		else:
 			return None
 	
+	def is_override(self):
+		return self.override and SnippetsLibrary().overridden[self.override]
+
 	def _override(self):
 		# Find the user file
 		target = SnippetsLibrary().get_user_library(self.language())
@@ -544,7 +565,7 @@ class SnippetsUserFile(SnippetsSystemFile):
 			sys.stderr.write("Error in making dirs\n")
 
 		try:
-			et.ElementTree(self.root).write(self.path, 'utf-8')
+			write_xml(self.root, self.path, ('text', 'accelerator'))
 			self.tainted = False
 		except IOError:
 			# Couldn't save, what to do
@@ -558,8 +579,8 @@ class SnippetsUserFile(SnippetsSystemFile):
 class SnippetsLibraryImpl:
 	def __init__(self):
 		self._accelerator_activated_cb = None
-		
 		self.loaded = False
+		self.check_buffer = gtk.TextBuffer()
 			
 	def set_dirs(self, userdir, systemdirs):
 		self.userdir = userdir
@@ -652,10 +673,14 @@ class SnippetsLibraryImpl:
 		revertto = self.overridden[snippet.override]
 		del self.overridden[snippet.override]
 		
-		snippet.revert(revertto)
+		if revertto:
+			snippet.revert(revertto)
+		
+			if revertto.id:
+				self.loaded_ids.append(revertto.id)
 	
 	def remove_snippet(self, snippet):
-		if not snippet.can_modify() or snippet.override:
+		if not snippet.can_modify() or snippet.is_override():
 			return
 		
 		# Remove from the library
@@ -808,7 +833,7 @@ class SnippetsLibraryImpl:
 		# Ensure language as well as the global snippets (None)
 		for lang in (None, language):
 			if lang in self.libraries:
-			# Ensure the container exists
+				# Ensure the container exists
 				self.container(lang)
 
 				for library in self.libraries[lang]:
@@ -828,6 +853,32 @@ class SnippetsLibraryImpl:
 
 		self.loaded = True
 
+	def valid_accelerator(self, keyval, mod):
+		mod &= gtk.accelerator_get_default_mod_mask()
+	
+		return (mod and (gdk.keyval_to_unicode(keyval) or \
+				keyval in range(gtk.keysyms.F1, gtk.keysyms.F12 + 1)))
+	
+	def valid_tab_trigger(self, trigger):
+		if not trigger:
+			return True
+
+		if trigger.isdigit():
+			return False
+
+		self.check_buffer.set_text(trigger)
+
+		start, end = self.check_buffer.get_bounds()
+		text = self.check_buffer.get_text(start, end)
+				
+		s = start.copy()
+		e = end.copy()
+		
+		end.backward_word_start()
+		start.forward_word_end()
+		
+		return (s.equal(end) and e.equal(start)) or (len(text) == 1 and not (text.isalnum() or text.isspace()))
+
 	# Snippet getters
 	# ===============
 	
